% NEO_IADJ_UTILIZ.MOD
% Neoclassical model with government consumption and investment spending financed by lump sum taxes
% Valerie Ramey, revised June 19,2020
% Corrected June 4, 2021 to insert missing u in MRS condition (did it by using easier specification of y/n)

% Baseline model, quarterly, with potential appropriation delays and/or time to build
% Includes investment adjustment costs, variable utilization

% Parameter calibration can be changed in Section 2. CALIBRATION: PARAMETER VALUES AND DEFINITIONS
% Type of government spending shock and whether there are delays can be chosen
%   in Options A, B, and C in Section 4. MODEL 

%----------------------------------------------------------------
% 0. HOUSEKEEPING
%----------------------------------------------------------------

close all;

%----------------------------------------------------------------
% 1. VARIABLES AND PARAMETERS
%----------------------------------------------------------------

% VARIABLE DEFINITIONS

%   y = output
%   c = consumption
%   i = investment
%   k = capital stock, end of period
%   q = Brainard-Tobin's q
%   u = capacity utilization
%   n = hours
%   z = labor-augmenting technological progress
%   gc = government consumption purchases
%   gi = government purchases = investment in public capital  
%   kg = public capital, end of period
%   util = utility of representative household
%   apport = exogenous apportionments of spending

%  ENDOGENOUS VARIABLES

var apport y c k i q u n gc gi kg ;  

% EXOGENOUS VARIABLES

varexo e;

% PARAMETERS

parameters beta nu phi delta deltag alpha z kappa thetag rhog delta1 delta2;  
parameters ky_ss iy_ss cy_ss n_ss y_ss gcy_ss giy_ss kgy_ss gc_ss gi_ss;
parameters q_ss u_ss;
parameters giy_opt_ss kgy_opt_ss; 

%----------------------------------------------------------------
% 2. CALIBRATION: PARAMETER VALUES AND DEFINITIONS
%----------------------------------------------------------------

alpha   = 0.36;            %  exponent on capital
beta    = 0.99;            %  discount factor
delta   = 0.015;           %  depreciation rate on capital
deltag  = 0.01;            %  depreciation rate on public capital
nu      = 1;               %  parameter on labor disutility
phi     = 0.25;            %  inverse Frisch elasticity
kappa   = 5.2;             % investment adj. cost, 5.2 from Leeper et al. (similar to SGU 2012) 
rhog    = 0.95;            % AR(1) on government spending/apportionments
z       = 1;               % Labor-augmenting technology
thetag  = 0.05;            % public capital prod. fn exponent
gammag  =  0.175;          % Steady-state G/Y, 2019 level
delta1  = 1/beta-1+delta;  % utiliz. cost linear term, so u = 1 in SS
delta2  = 2*delta1;        % utiliz. cost quadratic term, SGU 2.02 based on ACEL, Leeper et al. approx. 2

%----------------------------------------------------------------
% 3. STEADY-STATE VALUES
%----------------------------------------------------------------

ky_ss = alpha/(1/beta - 1 + delta); // SS capital-output ratio
iy_ss = delta*ky_ss; // SS investment-output ratio

kgy_opt_ss = thetag/(1/beta - 1 + deltag); // SS public-capital if public capital chosen optimally
giy_opt_ss = deltag*kgy_opt_ss;  //SS public investment/y if chosen optimally

%-----------------------------------------------
% STEADY-STATE PUBLIC INVESTMENT OPTION  ***

%  giy_ss = giy_opt_ss;  // set giy_ss at social optimum
   giy_ss = 0.035;   // set giy_ss at 2019 level
%------------------------------------------------

gcy_ss = gammag - giy_ss; // SS govt consumption-output ratio
kgy_ss = giy_ss/deltag;  // SS public capital-output ratio
cy_ss = 1 - iy_ss - gcy_ss - giy_ss;  // SS consumption-output ratio 

n_ss = ((1-alpha)/(nu*cy_ss))^(1/(1+phi)); // SS hours
y_ss = ((ky_ss^alpha)*((z*n_ss)^(1-alpha))*((giy_ss/deltag)^thetag))^(1/(1-alpha-thetag)); // SS output

c_ss = cy_ss*y_ss;
k_ss = ky_ss*y_ss;
kg_ss = kgy_ss*y_ss;
gc_ss = gcy_ss*y_ss;
gi_ss = giy_ss*y_ss;
i_ss = iy_ss*y_ss;
q_ss = 1;
u_ss = 1;

%util_ss = log(c_ss) - nu*(n_ss^(1+phi))/(1+phi);

%----------------------------------------------------------------
% 4. MODEL
%----------------------------------------------------------------

model; 

%----------------------------------------------------------------
% Expressions for use in model equations
%----------------------------------------------------------------

#adj = (kappa/2)*(i/i(-1) - 1)^2;
#adjdif = kappa*(i/i(-1) - 1);  % *** eliminated *(1/i(-1)) term
#adjdifp = kappa*(i(+1)/i - 1);  % *** eliminated *(1/i) term
#au = delta + delta1*(u - 1) + (delta2/2)*(u - 1)^2;
#aup = delta + delta1*(u(+1 )- 1) + (delta2/2)*(u(+1) - 1)^2;
#audif = delta1 + delta2*(u-1);


% 1) CONSUMPTION EULER EQUATION 

 (1/c)*q = beta*(1/c(+1))*(alpha*y(+1)/k + q(+1)*(1-aup));

% 2) INVESTMENT ADJUSTMENT COST

  (1/c) = (1/c)*q*(1 - adj - (i/i(-1))*adjdif) + beta*(1/c(+1))*q(+1)*(i(+1)/i)^2*adjdifp;

% 3) OPTIMAL CAPITAL UTILIZATION

  alpha*y/u = audif*q*k(-1);

% 3) MRS CONDITION

%  c*nu*n^phi = (1-alpha)*(kg(-1)^thetag)*(k(-1)^alpha)*(z^(1-alpha))*(n^(-alpha));
  c*nu*n^phi = (1-alpha)*y/n;

% 4) RESOURCE CONSTRAINT

   y = c + i + gc + gi ;  

% 5) PRODUCTION FUNCTION

  y = (u*k(-1))^alpha*(z*n)^(1-alpha)*kg(-1)^thetag;

% 6) PRIVATE CAPITAL ACCUMULATION

  k = (1-au)*k(-1) + i*(1-adj);

% 7) APPORTIONMENTS

  apport = rhog*apport(-1) + e;  

% 8) - 10) GOVERNMENT SPENDING AND PUBLIC CAPITAL ACCUMULATION PROCESSES 

% Choose one option and comment out the other two options


% OPTION A: SHOCK TO GOVERNMENT CONSUMPTION
   gc = gc_ss + apport; // Shock to government consumption
   gi = gi_ss; // No shock to government investment
   kg = gi_ss + (1-deltag)*kg(-1);  // No shock to government investment

/*
% OPTION B: SHOCK TO GOVERNMENT INVESTMENT, NO DELAYS
   gc = gc_ss;  // No shock to government consumption
   gi = gi_ss  + apport; // Shock to government investment, no delays
   kg =  gi_ss + (1-deltag)*kg(-1) + apport;    // Shock to government investment, no extra time to build
*/

/*
% OPTION C: SHOCK TO GOVERNMENT INVESTMENT, TIME-TO-SPEND AND TIME-TO-BUILD DELAYS
  gc = gc_ss;  // No shock to government consumption
  gi = gi_ss + (1/6)*(apport(-1) + apport(-2) + apport(-3) + apport(-4) + apport(-5) + apport(-6)); // time-to-spend
  kg =  gi_ss + (1-deltag)*kg(-1) + apport(-6);    // shock to government investment, 6-quarter time to build
*/


% 11) UTILITY

% util = ln(c) - nu*(n^(1+phi))/(1+phi);

end;

%----------------------------------------------------------------
% 5. COMPUTATION
%----------------------------------------------------------------

initval;
  y = y_ss;
  c = cy_ss*y_ss;
  k = k_ss;
  i = i_ss;
  q = q_ss;
  u = u_ss;
  n = n_ss;
  gc = gcy_ss*y_ss;
  gi = giy_ss*y_ss;
  kg = kgy_ss*y_ss;
end;

 check;
 steady; 

 shocks;
 var e;
 periods 1;
 values (.01*y_ss);
 end;

 simul(periods=1000);

%A = [oo_.exo_simul, oo_.endo_simul'];
A = [oo_.exo_simul, apport, y, c, i, n, u, k, kg, gc, gi] ;

%% To print out results (with lots of significant figures) type the following in matlab
%% A=[oo_.exo_simul, oo_.endo_simul']
%% Then double-click A matrix in upper right to get it to print it with significant figures
%%  and then copy to Excel