%--------------------------------------------------------------------------
% NK_IADJ_UTILIZ.MOD
% By Valerie Ramey, revised May 12, 2021 to correct "transfers" to "taxes" in June 20, 2020 version

% Medium-scale TANK model with government investment
% Contains:
%   - public capital
%   - rule-of-thumb consumers
%   - sticky prices and sticky wages with possible indexing
%   - adjustment cost on investment rather than on capital
%   - variable utilization as in SGU
% Based on extensions of Sarah Zubairy program, Martin Uribe notes, Schmitt-Grohe-Uribe (2005) (SGU0, Colciago (2011)

% Parameter calibration can be changed in Section 2. CALIBRATION: PARAMETER VALUES AND DEFINITIONS
% Type of government spending shock and whether there are delays can be chosen
%   in Options A, B, and C in Section 4. MODEL 
%--------------------------------------------------------------------------

%----------------------------------------------------------------
% 0. HOUSEKEEPING
%----------------------------------------------------------------

close all;
clc;

%----------------------------------------------------------------
% 1. VARIABLES AND PARAMETERS
%----------------------------------------------------------------

% VARIABLE DEFINITIONS

%   c = consumption, total
%   co = consumption of optimizing agents
%   cr = consumption of rule-of-thumb agents

%   hd = labor hours demanded
%   h = hours, total supplied
%   ho = consumption of optimizing agents
%   hr = consumption of rule-of-thumb agents

%   iv = investment, total
%   ivo = investment of optimizing agents
%   k = capital stock, total, end of period
%   ko = capital of optimizing agents
%   q = Brainard-Tobin's q
%   u = capacity utilization

%   y = output
%   lambda = Lagrange multiplier on optimizing HH budget constraint
%   w = wage (real)
%   rk = rental price of capital (real)

%   gc = government consumption purchases
%   gi = government purchases = investment in public capital  
%   kg = public capital, end of period
%   apport = exogenous apportionments of spending
%   b = government debt
%   t = total taxes
%   to = taxes on optimizing HH
%   tr = taxes on rule-of-thumb HH

%   s = price distortion due to price stickiness, s=1 in ss, >1 out of ss
%   sw = wage distortion due to wage stickiness, sw=1 in ss, >1 out of ss
%   r = gross nominal interest rate
%   pi = gross inflation rate, p/p(-1)
%   p = relative price of price adjusters to final goods price
%   mc = marginal cost
%   wstar = wage set by wage adjusters
%   x1 = auxiliary term for price stickiness
%   x2 = auxiliary term for price stickiness
%   f1 = auxiliary term for wage stickiness
%   f2 = auxiliary term for wage stickiness

%  util = utility function of average household


%  ENDOGENOUS VARIABLES  (38 variables)

var co cr c;          % consumption 
var ho hr h hd;       % hours 
var ko k ivo iv q u;  % investment and capital stock, utilization
var y;                % output
var lambda w rk;      % Lagrange multiplier on HH budget constraint, wages, rental rate of capital
var p mc muw;         % relative price, marginal cost, wage markup 
var s sw wstar ;      % price distortion( due to price stickiness), wage distortion, reset wage 
var x1 x2 f1 f2;      % auxiliary terms for price and wage stickiness
var r pi;             % gross nominal interest rate,inflation
var t tr to b;        % total taxes, taxes on ROT HH, taxes on optimizing HH, govt debt  
var gi gc kg g apport;       % government investment, public capital, government consumption
var util;

% EXOGENOUS VARIABLES (1 variable)

varexo e;        % shock to apportionment
 

%----------------------------------------------------------------
% 2. PARAMETERS AND CALIBRATION
%----------------------------------------------------------------

% PARAMETERS

parameters beta;                     % discount factor
parameters delta;                    % depreciation rate on private capital
parameters deltag;                   % depreciation rate on government capital
parameters kappa;                    % investment adjustment cost
parameters delta1;                   % parameter of utilization cost
parameters delta2;                   % parameter of utilization cost
parameters alpha;                    % capital share
parameters gamma;                    % rule-of-thumb share
parameters theta;                    % price stickiness
parameters thetaw;                   % wage stickiness
parameters chi;                      % degree of wage indexing
parameters phipi;                    % monetary policy
parameters phig phib;                % fiscal financing parameters
parameters eps;                      % elasticity of substitution of goods varieties
parameters epsw;                     % elasticity of substitution of labor varieties
parameters muw_ss;                   % steady-state wage markup
parameters mup_ss;                   % steady-state price markup
parameters phi;                      % utility parameter (1/phi is Frisch elasticity)
parameters nu;                       % weight on disutility of labor
parameters thetag;                   % public capital prod. fn exponent
parameters gammag;                   % G/Y fraction
parameters rhog;                     % AR(1) on government spending

%Steady state parameters

parameters co_ss cr_ss c_ss;
parameters ho_ss hr_ss h_ss hd_ss ko_ss k_ss ivo_ss iv_ss q_ss u_ss;
parameters lambda_ss w_ss rk_ss s_ss sw_ss r_ss pi_ss y_ss;
parameters t_ss tr_ss to_ss p_ss b_ss mc_ss wstar_ss x1_ss x2_ss f1_ss f2_ss g_ss;
parameters gc_ss gi_ss kg_ss;
parameters giy_ss kgy_ss giy_opt_ss kgy_opt_ss;


delta    = .015;          %.015  depreciation rate of private capital; GLV = 0.025
deltag   = .01;           % .01 depreciation rate of public capital;
kappa    = 5.2;           % investment adj. cost, 5.2 from Leeper et al. (similar to SGU 2012)
phi      = 0.25;          % utility parameter on hours (1/phi is Frisch elasticity) GLV =  0.2, I use 0.25
alpha    = 0.36;          % capital share; GLV = 1/3
mup_ss   = 1.2;           % steady state price markup
muw_ss   = 1.2;           % wage markup
theta    = 0.75;          % price stickiness, 0 implies no stickiness; GLV = 0.75
thetaw   = 0.75;          % wage stickiness, 0 implies no stickiness
chi      = 0;             % wage indexing, , 0=no, 1=full
gamma    = 0.5;           % rule-of-thumb share, GLV = 0.5
beta     = .99;           % discount factor
phib     = 0.33;          % fiscal rule parameters, GLV = 0.33 
phig     = 0.1;           % fiscal rule parameters, lower phig means less increase in tax, GLV = 0.1 
phipi    = 1.5;           % monetary rule parameter
eps      = mup_ss/(mup_ss-1);  % elasticity of subsitution of goods varieties
epsw     = muw_ss/(muw_ss-1);  % elasticity of substitution of labor varieties
pi_ss    = 1;             % steady state inflation
thetag   = 0.05;          % share of public capital in production
nu       = 1;             % weight on disutility of labor
gammag  =  0.175;   //Steady-state G/Y, 2019 level
rhog     = 0.95;          % AR(1) on government spending
delta1   = 1/beta-1+delta;% utiliz. cost linear term, so u = 1 in SS
delta2   = 2*delta1;      % utiliz. cost quadratic term, SGU 2.02 based on ACEL, Leeper et al. approx. 2

%----------------------------------------------------------------
% 3. STEADY-STATE VALUES OF ENDOGENOUS VARIABLES
%----------------------------------------------------------------

% Note that the order of these matter.

kgy_opt_ss = thetag/(1/beta - 1 + deltag); // SS public-capital-output ratio if public capital chosen optimally
giy_opt_ss = deltag*kgy_opt_ss;  // SS public-investment-to-output if chosen optimally
%       giy_ss = giy_opt_ss;          // set giy_ss at social optimum ***
giy_ss = 0.035;            // set giy_ss at 2019 level
gcy_ss = gammag - giy_ss;        // total g/y = 0.2
kgy_ss = giy_ss/deltag;       // SS public capital-output ratio  

s_ss = 1;
p_ss = 1;
mc_ss = 1/mup_ss;
rk_ss = 1/beta -1+delta;
sw_ss = 1;
u_ss = 1;

ky_ss = alpha/((1/beta - 1 + delta)*mup_ss); 
iy_ss = delta * ky_ss;
gy_ss = gammag;  //**
cy_ss = 1 - iy_ss - gy_ss; //**
h_ss = ((1-alpha)/(mup_ss*muw_ss*nu*cy_ss))^(1/(1+phi)); 
hd_ss = h_ss;
y_ss = ((ky_ss^alpha)*(hd_ss^(1-alpha))*((giy_ss/deltag)^thetag))^(1/(1-alpha-thetag)); 
k_ss = y_ss* ky_ss;
w_ss = (k_ss/hd_ss)*rk_ss*(1-alpha)/alpha;
wstar_ss = w_ss;
c_ss = cy_ss*y_ss; 
ho_ss = h_ss;
hr_ss = h_ss;
ko_ss = k_ss/(1-gamma);
iv_ss = delta*k_ss;
ivo_ss = iv_ss/(1-gamma);
q_ss = 1;
co_ss = c_ss;
cr_ss = c_ss;
g_ss = y_ss-iv_ss-c_ss;
gi_ss = giy_ss*y_ss;
gc_ss = g_ss - gi_ss;
kg_ss = 1/deltag*gi_ss;
tr_ss = w_ss*hr_ss-cr_ss;
r_ss = 1/beta;
b_ss = 0;
to_ss = (g_ss-gamma*tr_ss)/(1-gamma);
t_ss = (1-gamma)*to_ss + gamma*tr_ss;
x1_ss = y_ss/(1-theta*beta);
x2_ss = x1_ss;
lambda_ss = 1/co_ss;
f1_ss = w_ss*lambda_ss*hd_ss*muw_ss^(-1)/(1-thetaw*beta);
f2_ss = f1_ss;


%----------------------------------------------------------------
% MODEL EQUATIONS   (36 equations)
%----------------------------------------------------------------

model; %(bytecode, block);

%----------------------------------------------------------------
% Expressions for use in model equations
%----------------------------------------------------------------

#adj = (kappa/2)*(ivo/ivo(-1) - 1)^2;
#adjdif = kappa*(ivo/ivo(-1) - 1);
#adjdifp = kappa*(ivo(+1)/ivo - 1);
#au = delta + delta1*(u - 1) + (delta2/2)*(u - 1)^2;
#aup = delta + delta1*(u(+1 )- 1) + (delta2/2)*(u(+1) - 1)^2;
#audif = delta1 + delta2*(u-1);


%----------------------------------------------------------------
% Model equations
%----------------------------------------------------------------

% 1) Capital equation of motion

  ko = (1-au)*ko(-1) + ivo*(1-adj);

% 2) SHADOW VALUE OF WEALTH

  lambda = 1/co;

% 3) CONSUMPTION EULER EQUATION WITH TOBIN'S Q 

 lambda*q = beta*lambda(+1)*(rk(+1)*u(+1) + q(+1)*(1-aup));

% 4) INVESTMENT ADJUSTMENT COST

  lambda = lambda*q*(1 - adj - (ivo/ivo(-1))*adjdif) + beta*lambda(+1)*q(+1)*(ivo(+1)/ivo)^2*adjdifp;

% 5) OPTIMAL CAPITAL UTILIZATION

  rk = q*audif;

% 6) RULE-OF-THUMB CONSUMPTION

  cr = w*hr - tr;

% 7) MRS CONDITION

  muw = w*(gamma/(cr*nu*hr^phi) + (1-gamma)/(co*nu*ho^phi));

% 8) EQUALITY OF HOURS ACROSS TYPES

  ho = hr;

% 9)-11) SGU RECURSIVE EQS FOR OPTIMAL WAGE SETTING - TANK VERSION (substitutes gamma/cr + (1-gamma)/co for lambda in f1)

  f1 = (epsw-1)/epsw*wstar*(gamma/cr + (1-gamma)/co)*(w/wstar)^epsw*hd + thetaw*beta*(pi(+1)/pi^chi)^(epsw-1)*(wstar(+1)/wstar)^(epsw-1)*f1(+1);  
  f2 = nu*h^phi*(w/wstar)^epsw*hd + thetaw*beta*(pi(+1)/pi^chi)^epsw*(wstar(+1)/wstar)^epsw*f2(+1); 
  f1 = f2;

% 12) - 13) DISTORTIONS DUE TO STICKY WAGES

  h = sw*hd;
  sw = (1-thetaw)*(wstar/w)^(-epsw) + thetaw*(w(-1)/w)^(-epsw)*(pi/pi(-1)^chi)^epsw*sw(-1);

% 14) AGGREGATE WAGES

  w^(1-epsw) = (1-thetaw)*wstar^(1-epsw) + thetaw*w(-1)^(1-epsw)*(pi(-1)^chi/pi)^(1-epsw);

% 15) FOC FOR FIRM OPTIMAL CAPITAL-LABOR RATIO

  u*k(-1)/hd = alpha/(1-alpha)*w/rk;  %  ** correction - included u

% 16) REAL MARGINAL COST

  mc = rk^alpha*w^(1-alpha)*alpha^(-alpha)*(1-alpha)^(-1+alpha)*kg(-1)^(-thetag); % correction - excluded u

% 17) - 19) URIBE RECURSIVE EQS FOR OPTIMAL PRICE SETTING (SGU 2005 SEEM DIFFERENT)

  x1 = mup_ss*p^(-eps)*y*mc + theta*beta*lambda(+1)/lambda*x1(+1);
  x2 = p^(1-eps)*y + theta*beta*lambda(+1)/lambda*p/p(+1)*x2(+1)/pi(+1);
  x1 = x2;

% 20) AGGREGATE INFLATION

  1 = (1-theta)*p^(1-eps) + theta*pi^(eps-1);

% 21) AGGREGATE CONSUMPTION

  c = gamma*cr + (1-gamma)*co;

% 22) AGGREGATE HOURS

  h = gamma*hr + (1-gamma)*ho;

% 23) AGGREGATE INVESTMENT

  ivo=iv/(1-gamma);

% 24) AGGREGATE CAPITAL

  ko=k/(1-gamma);

% 25) AGGREGATE TAXES

  t = gamma*tr + (1-gamma)*to;

% 26) RESOURCE CONSTRAINT

  y = c + iv + g;

% 27) PRODUCTION FUNCTION (S>=1 CAPTURES DISTORTION DUE TO STICKY PRICES)

  s*y = kg(-1)^thetag*(u*k(-1))^alpha*hd^(1-alpha); % include public capital

% 28) EQUATION OF MOTION FOR STICKY PRICE DISTORTION

  s = (1-theta)*p^(-eps) + theta*pi^eps*s(-1);

% 29) GROSS NOMINAL INTEREST RATE

  lambda = r*beta*lambda(+1)/pi(+1);

% 30) TAYLOR RULE

  (r-r_ss) = phipi*(pi-pi_ss);

% 31) GOVERNMENT BUDGET CONSTRAINT

  g = b/r - b(-1)/pi + t;

% 32) - 33) TAX RULES

  t = t_ss + phib*(b(-1) - b_ss) + phig*(g - g_ss);
  to = to_ss + tr - tr_ss;  // Uribe says that GLV implicitly assume this

% 34) and 35) APPORTIONMENTS AND DEFINITION OF TOTAL GOVERENMENT SPENDING

  apport = rhog*apport(-1) + e;

  g = gc + gi;

% 36) - 38) GOVERNMENT SPENDING AND PUBLIC CAPITAL ACCUMULATION PROCESSES 

% Choose one option and comment out the other two options


% OPTION A: SHOCK TO GOVERNMENT CONSUMPTION
   gc = gc_ss + apport; // Shock to government consumption
   gi = gi_ss; // No shock to government investment
   kg = gi_ss + (1-deltag)*kg(-1);  // No shock to government investment

/*
% OPTION B: SHOCK TO GOVERNMENT INVESTMENT, NO DELAYS
   gc = gc_ss;  // No shock to government consumption
   gi = gi_ss  + apport; // Shock to government investment, no delays
   kg =  gi_ss + (1-deltag)*kg(-1) + apport;    // Shock to government investment, no extra time to build
*/

/*
% OPTION C: SHOCK TO GOVERNMENT INVESTMENT, TIME-TO-SPEND AND TIME-TO-BUILD DELAYS
  gc = gc_ss;  // No shock to government consumption
  gi = gi_ss + (1/6)*(apport(-1) + apport(-2) + apport(-3) + apport(-4) + apport(-5) + apport(-6)); // time-to-spend
  kg =  gi_ss + (1-deltag)*kg(-1) + apport(-6);    // shock to government investment, 6-quarter time to build
*/

% 39) UTILITY

  util = gamma*(ln(cr) - nu*(hr^(1+phi))/(1+phi)) + (1-gamma)*(ln(co) - nu*(ho^(1+phi))/(1+phi));

end;

%----------------------------------------------------------------
% STEADY-STATE
%----------------------------------------------------------------

initval;
  co = co_ss;
  cr = cr_ss;
  c = c_ss;
  ho = ho_ss;
  hr = hr_ss;
  h = h_ss;
  k = k_ss;
  ko = ko_ss;
  ivo = ivo_ss;
  iv = iv_ss;
  q = q_ss;
  u = u_ss;
  lambda = lambda_ss;
  w = w_ss;
  rk = rk_ss;
  s = s_ss;
  r = r_ss;
  pi = pi_ss;
  y = y_ss;
  to = to_ss;
  tr = tr_ss;
  t = t_ss;
  p = p_ss;
  b = b_ss;
  mc = mc_ss; 
  x1 = x1_ss; 
  x2 = x2_ss; 
  kg= kg_ss;
  gi= gi_ss;
  gc=gc_ss;
  wstar = wstar_ss;
  f1 = f1_ss;
  f2 = f2_ss;
  muw = muw_ss;
  hd = hd_ss;
  sw = sw_ss;
end;

 check;
 steady; 

shocks;
 var e;
 periods 1;
 values (.01*y_ss);
end;

 simul(periods=1000);

% A = [oo_.exo_simul, oo_.endo_simul'];
A = [oo_.exo_simul, apport, y, c, co, cr, iv, h, u, w, rk, r, pi, k, kg, gc, gi] ;



